<?php
/*
Plugin Name: DH Local SEO
Plugin URI: http://www.digitaleheimat.de
Description: DH Local SEO improves your visibility by adding location information in Schema.org format to your posts and creates a sitemap with geo information.
Version: 1.0
Author: digitaleheimat GmbH
Author URI: http://www.digitaleheimat.de
License: GPLv2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.html
*/


class DHLocalSeo {

    public $fields;
    
    
    /**
     * __construct
     * Constructor. Sets variables for fields and default settings, adds basic action and filter hooks
     *
     * @since 1.0.
     *
     */ 
    function __construct () {
        
        $this->fields = (object) array(
                'name' => '_dhl_name',
                'streetAddress' => '_dhl_streetAddress',
                'postalCode' => '_dhl_postalCode',
                'addressLocality' => '_dhl_addressLocality',
                'latitude' => '_dhl_latitude',
                'longitude' => '_dhl_longitude',
                'telephone' => '_dhl_telephone',
                'faxNumber' => '_dhl_faxNumber',
                'email' => '_dhl_email',
                'url' => '_dhl_url',
                'openingHours' => '_dhl_openingHours',        
                'description' => '_dhl_description',        
         );
         
        $defaults = array(
                'gmaps_api_key' => '',   
                'enable_location' => true,
                'enable_post_types' => array(),
                'enable_telephone' => true,
                'enable_faxNumber' => true,
                'enable_email' => true,
                'enable_url' => true,
                'enable_description' => true,
                'enable_openingHours' => true,
                //'enable_sitemap' => false,
                //'enable_wpseo_support' => true,        
                'set_the_content_filter' => true,        
                'set_the_content_filter_position' => 10,        
        );

        
        $this->settings = (object) $defaults;

         
        //general init
        add_action( 'init', array($this, 'init') );
        

        add_action( 'template_redirect', array( $this, 'template_redirect' )  );
        
        //add meta boxes
        add_action( 'add_meta_boxes', array($this, 'add_meta_boxes') );
        //backwards compatibility
        add_action( 'admin_init', array($this, 'add_meta_boxes'), 1 );
        
        //admin init
        add_action( 'admin_init', array($this, 'admin_init'), 1 );
        
        //admin menu
        add_action( 'admin_menu', array($this, 'admin_menu'), 1 );
                
        //save meta boxes
        add_action( 'save_post', array($this, 'save_meta_boxes') ); 

        //textdomain
        add_action( 'plugins_loaded', array($this, 'load_plugin_textdomain') );  
        
        //add shortcode
        add_shortcode( 'dh_localseo', array($this, 'shortcode') );    
 
    }

    /**
     * init
     * Initializes the Plugin: Loads settings, creates custom post type "location", enqueues scripts and styles
     *
     * @since 1.0.
     *
     */    
    function init() {
        
        if (get_option('dh_localseo'))
            $this->settings = (object) get_option('dh_localseo');
        
        if ($this->settings->enable_location)
            register_post_type( 'location',
                array(
                    'labels' => array(
                        'name' => __( 'Locations', 'dh_localseo'),
                        'singular_name' => __( 'Location', 'dh_localseo' )
                    ),
                'public' => true,
                'supports' => array('title','author')
                )
            );
        
        //rewrite rules
        $this->add_rewrite_rules();

        wp_enqueue_script('gmaps', 'https://maps.googleapis.com/maps/api/js?key='.$this->settings->gmaps_api_key.'&sensor=false');
        wp_enqueue_script('dh-localseo', plugins_url('js/dh-localseo.js', __FILE__), array('gmaps', 'jquery'));
        
        wp_enqueue_style('dh-localseo',plugins_url('css/dh-localseo.css', __FILE__));
        
        if($this->settings->set_the_content_filter) {
            $filter_position =  $this->settings->set_the_content_filter_position;
            if ((int)$filter_position == 0) //save cast
                $filter_position = 10; 
            add_filter( 'the_content', array($this, 'the_content'), $filter_position );          
        }
    }
    /**
     * load_plugin_textdomain
     * Loads textdomain
     *
     * @since 1.0.
     *
     */     
    function load_plugin_textdomain() {
      load_plugin_textdomain( 'dh_localseo', false, dirname( plugin_basename( __FILE__ ) )  . '/languages/' ); 
    }
    
    /**
     * template_redirect
     * redirects to sitemap / kml generator
     *
     * @since 1.0.
     *
     */ 
	function template_redirect() {
        if (get_query_var( 'dhl_sitemap' ))
            $this->create_sitemap();        
        if (get_query_var( 'kml' ))
            $this->create_kml();
    }    

    /**
     * options_page
     * creates the settings page
     *
     * @since 1.0.
     *
     */

    function options_page() {
?>
<div class="wrap">
  <form method="post" action="options.php">
  <h2><?php _e('DH Local SEO settings', 'dh_localseo'); ?></h2>
            <?php settings_fields( 'dh_localseo_settings' ); ?>
            <?php do_settings_sections( 'dh-localseo-settings' ); ?>
            <?php submit_button(); ?>
<?php    
  
    }
    
    /**
     * admin_init 
     * inits the plugin in admin interface
     * @since 1.0.
     *
     */

     
    function admin_init() {
        
        //init settings for settings page
            
        //registers the settings
        register_setting( 'dh_localseo_settings', 'dh_localseo' );
        add_option('dh_localseo', (array) $this->settings);
        add_settings_section('maps_settings', __('Google Maps settings', 'dh_localseo'), null, 'dh-localseo-settings');
        add_settings_section('post_type_settings', __('Post type settings', 'dh_localseo'), null, 'dh-localseo-settings');
        add_settings_section('additional_settings', __('Enable additional fields', 'dh_localseo'), array($this,'additional_settings'), 'dh-localseo-settings');
        add_settings_section('sitemap_settings', __('GEO sitemap settings', 'dh_localseo'), array($this,'sitemap_settings'), 'dh-localseo-settings');
        add_settings_section('template_settings', __('Template settings', 'dh_localseo'), array($this,'template_settings'), 'dh-localseo-settings');
        
        //set all the fields
        add_settings_field('gmaps-api-key', __('Google Maps API Key', 'dh_localseo'), array($this, 'input_text'), 'dh-localseo-settings', 'maps_settings', array('name' => 'dh_localseo[gmaps_api_key]', 'value' => $this->settings->gmaps_api_key, 'help' => __('An API console key is recommended and provided by <a href="https://developers.google.com/maps/documentation/javascript/tutorial?hl=en#api_key" target="_blank">Google</a>.', 'dh_localseo')));
        
        add_settings_field('enable-location', __('Enable "Location" as post type', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'post_type_settings', array('name' => 'dh_localseo[enable_location]', 'value' => $this->settings->enable_location, 'help' => __('enables a simple post type "Location" with only location information.', 'dh_localseo')));        
        
        // get other existing post_types
        foreach (get_post_types(array('public' =>true, 'show_ui' => true), 'object') as $type) {
            if ($type->name == 'location' || $type->name == 'attachment')
                continue;
            add_settings_field('enable-post-types-'.$type->name, sprintf(__('Enable for post type "%s"', 'dh_localseo'), $type->labels->name), array($this, 'input_checkbox'), 'dh-localseo-settings', 'post_type_settings', array('name' => 'dh_localseo[enable_post_types]['.$type->name.']', 'value' => $this->settings->enable_post_types[$type->name], 'help' => sprintf(__('Shows location information fields for post type "%s".', 'dh_localseo'), $type->labels->name)));                
        }
        
        
        add_settings_field('enable-telephone', __('Phone', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'additional_settings', array('name' => 'dh_localseo[enable_telephone]', 'value' => $this->settings->enable_telephone, 'help' => __('Enables a text field to enter the phone number', 'dh_localseo')));
        add_settings_field('enable-faxNumber', __('Fax', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'additional_settings', array('name' => 'dh_localseo[enable_faxNumber]', 'value' => $this->settings->enable_faxNumber, 'help' => __('Enables a text field to enter the fax number', 'dh_localseo')));
        add_settings_field('enable-email', __('Email address', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'additional_settings', array('name' => 'dh_localseo[enable_email]', 'value' => $this->settings->enable_email, 'help' => __('Enables a text field to enter the email address; <code>mailto:</code> link in default template', 'dh_localseo')));
        add_settings_field('enable-url', __('Website', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'additional_settings', array('name' => 'dh_localseo[enable_url]', 'value' => $this->settings->enable_url, 'help' => __('Enables a text field to enter the website; linked in default template', 'dh_localseo')));
        add_settings_field('enable-description', __('Description', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'additional_settings', array('name' => 'dh_localseo[enable_description]', 'value' => $this->settings->enable_description, 'help' => __('Enables a textarea to enter a description with html allowed and automatic linebreaks', 'dh_localseo')));
        add_settings_field('enable-openingHours', __('Opening hours', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'additional_settings', array('name' => 'dh_localseo[enable_openingHours]', 'value' => $this->settings->enable_openingHours, 'help' => __('Enables a dynamic fieldset to enter various opening hours; autosort by weekday and time', 'dh_localseo')));
        
        //add_settings_field('enable-sitemap', __('Enable geo sitemap', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'additional_settings', array('name' => 'dh_localseo[enable_sitemap]', 'value' => $this->settings->enable_sitemap/*, 'help' => 'enables a simple post type "Location" with only location information.'*/));
        //add_settings_field('enable-wpseo-support', __('Enable support for WordPress SEO', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'sitemap_settings', array('name' => 'dh_localseo[enable_wpseo_support]', 'value' => $this->settings->enable_wpseo_support, 'disabled' => !is_plugin_active('wordpress-seo/wp-seo.php'), 'help' => 'If you are using <a href="http://yoast.com/wordpress/seo/#utm_source=wpadmin&utm_medium=plugin&utm_campaign=wpseoplugin" target="_blank">WordPress SEO</a>, you can let the sitemap be automatically integrated in the generated sitemap index.'));
        
        add_settings_field('set-the_content-filter', __('Append template automatically', 'dh_localseo'), array($this, 'input_checkbox'), 'dh-localseo-settings', 'template_settings', array('name' => 'dh_localseo[set_the_content_filter]', 'value' => $this->settings->set_the_content_filter, 'help' => __('Automatically appends location information template to post content through <code>the_content</code> filter', 'dh_localseo')));
        add_settings_field('set-the_content-filter-position', __('Position in filter herarchy', 'dh_localseo'), array($this, 'input_text'), 'dh-localseo-settings', 'template_settings', array('name' => 'dh_localseo[set_the_content_filter_position]', 'value' => $this->settings->set_the_content_filter_position, 'help' => __('Adjusts, when within the filter the template is loaded; <strong>Note:</strong> Only necessary for ordering when several filters are active. Default: <code>10</code> ', 'dh_localseo')));

        
    } 

    
    /**
     * admin_menu 
     * adds admin menu page
     *
     * @since 1.0.
     *
     */

    function admin_menu() {
        add_options_page(__('DH Local Seo'), __('DH Local Seo'), 'manage_options', 'dh-localseo-settings', array($this, 'options_page'));   
    }
    
    /**
     * input_text 
     * template for text input fields
     *
     * @since 1.0.
     *
     */
   
    function input_text($args) {
        $name = esc_attr( $args['name'] );
        $value = esc_attr( $args['value'] );
        $help = html_entity_decode(esc_attr( $args['help'] ));     
        echo "<input type='text' name='$name' value='$value' class='regular-text' />";
        if ($help)
            echo "<span class='howto'>$help</span>";
    }

    
    /**
     * input_text 
     * template for checkbox input fields
     *
     * @since 1.0.
     *
     */
       
    function input_checkbox($args) {
        $name = esc_attr( $args['name'] );
        $value = esc_attr( $args['value'] );
        $help = html_entity_decode(esc_attr( $args['help'] ));        
        $disabled = ($args['disabled']) ? "disabled='disabled' " : '';        
        $checked = ($value && !$disabled) ? "checked='checked' " : '';
        echo "<input type='checkbox' name='$name' value='true' $checked$disabled/>";
        if ($help)
            echo "<span class='howto'>{$help}</span>";
       
    }    
  
  
    /**
     * additional_settings 
     * head info for additional settings
     *
     * @since 1.0.
     *
     */
       
    function additional_settings () {
           
?>
        <p><?php _e('The plugin supports, next to the default address fields, additional SEO relevant fields.', 'dh_localseo') ?></p>
        <p><?php _e('<strong>Note:</strong> The Schema.org notation is implemented in the default template and can be overridden through a custom template (see below).', 'dh_localseo') ?></p>
<?php    
    }

  
    /**
     * sitemap_settings 
     * head info for sitemap settings
     *
     * @since 1.0.
     *
     */    
    function sitemap_settings () {
        global $blog_id;
        $sitemap_url = home_url('geo-sitemap.xml');
       
?>
        <p><?php _e('The plugin automatically creates a sitemap with geo information that can ie. be submitted to Google Webmasters:', 'dh_localseo') ?> <a href="<?php echo $sitemap_url?>" target="_blank"><?php echo $sitemap_url?></a></p>
        <p><?php printf(__('<strong>Note:</strong> If the sitemap is not avaiable you might have to <a href="%s">flush the rewrite rules</a>.', 'dh_localseo'), get_admin_url($blog_id,'options-general.php?page=dh-localseo-settings&flush_rewrite_rules=1')); ?></p>
<?php      
    }

  
    /**
     * template_settings 
     * head info for template settings
     *
     * @since 1.0.
     *
     */    
    function template_settings () {
    
        if (!file_exists($template_path = get_stylesheet_directory().'/location-template.php')) {
            $template_path = plugin_dir_path(__FILE__).'/location-template.php';
            $template_info = sprintf(__('You are currently using the default template for displaying the location information. You can override the template in your theme folder. Just copy the default template from <code>%s</code>.'), $template_path);
        } else
            $template_info = sprintf(__('You are currently using a custom template for displaying the location information. To use the default, just delete the custom template from <code>%s</code>', 'dh_localseo'), $template_path);
        
?>
        <p><?php echo $template_info ?></p>
        <p><?php _e('The plugin can append the template automatically using <code>the_content</code> filter. You can also add the template to your theme manually using the function <code>print $dh_localseo->show_location_template()</code> or insert it directly into the post content using the shortcode <code>[dh_localseo]</code>.', 'dh_localseo') ?></p>
<?php
        
    }    
    
    /**
     * add_meta_boxes
     * Adds the Location information meta boxes to all activated post types
     *
     * @since 1.0.
     *
     */

    function add_meta_boxes() {     
        
        $screens = $this->get_enabled_post_types();
        
        if(!empty($screens)) foreach ($screens as $screen) {
            add_meta_box(
                'dh_localseo-meta-box',
                __( 'Location information', 'dh_localseo' ),
                array($this, 'show_meta_box'),
                $screen
            );
        }
    }

  
    /**
     * show_meta_box
     * Shows meta box form
     *
     * @since 1.0.
     *
     */
     
    function show_meta_box () {
        global $post;
        // Use nonce for verification
        wp_nonce_field( plugin_basename( __FILE__ ), 'dh_localseo_nonce' );

        // The actual fields for data entry
        // Use get_post_meta to retrieve an existing value from the database and use the value for the form
      
        foreach ($this->fields as $field => $name) {           
            $meta->$field = get_post_meta( $post->ID, $name, true );
 
        }
      
      
    ?>
        <table class="form-table">
            <tr valign="top">
                <th scope="row"><label for="_dhl_name"><?php _e('Name', 'dh_localseo');?></label></th>
                <td><input type="text" id="dhl_name" name="_dhl_name" value="<?php print $meta->name  ?>" class="regular-text" /><br/>
                <span class="howto"><?php _e('Name of the location', 'dh_localseo');?></span></td>
            </tr>        
            <tr valign="top">
                <th scope="row"><label for="_dhl_streetAddress"><?php _e('Street', 'dh_localseo');?></label></th>
                <td><input type="text" id="dhl_streetAddress" name="_dhl_streetAddress" value="<?php print $meta->streetAddress  ?>" class="regular-text" /><br/>
                <span class="howto"><?php _e('Street and no. of the location', 'dh_localseo');?></span></td>
            </tr>
            <tr valign="top">
                <th scope="row"><label for="_dhl_postalCode"><?php _e('Postal code and city', 'dh_localseo');?></label></th>
                <td><input type="text" id="dhl_postalCode" name="_dhl_postalCode" value="<?php print $meta->postalCode  ?>" style="width: 80px;" />  <input type="text" id="dhl_addressLocality" name="_dhl_addressLocality" value="<?php print $meta->addressLocality  ?>" style="width: 215px;" />  <br/>
                <span class="howto"><?php _e('Postal code and city', 'dh_localseo');?></span></td>
            </tr>    
            <tr valign="top">
                <th scope="row"><label><?php _e('Google map', 'dh_localseo');?></label></th>
                <td>
                <input type="submit"id="checkAddress" value="<?php _e('Check address', 'dh_localseo');?>"/><br/>
                 <span class="howto"><?php _e('Address needs to be checked to update Google Map data', 'dh_localseo');?></span>
                 <span class="howto invalid_geodata" style="display: none;color:#ff0000"><?php _e('Geodata was reset due to empty or invalid adress.', 'dh_localseo');?></span>
                <div id="map-canvas" style="width: 320px; height: 240px;"></div>
                <label><?php _e('Latitude', 'dh_localseo');?></label><input type="text" id="dhl_latitude" name="_dhl_latitude" value="<?php print $meta->latitude  ?>" style="width: 80px;" readonly="readonly"/>  <label><?php _e('Longitude', 'dh_localseo');?></label><input type="text" id="dhl_longitude" name="_dhl_longitude" value="<?php print $meta->longitude  ?>" style="width: 80px;" readonly="readonly"/>  <br/>
                <span class="howto"><?php _e('Data is retrieved automatically', 'dh_localseo');?></span></td>
            </tr>
            <?php if($this->settings->enable_telephone): ?>
            <tr valign="top">
                <th scope="row"><label for="_dhl_telephone"><?php _e('Phone', 'dh_localseo');?></label></th>
                <td><input type="text" id="dhl_telephone" name="_dhl_telephone" value="<?php print $meta->telephone  ?>" class="regular-text" /></td>
            </tr> 
            <?php endif; ?>
            <?php if($this->settings->enable_faxNumber): ?>            
            <tr valign="top">
                <th scope="row"><label for="_dhl_faxNumber"><?php _e('Fax', 'dh_localseo');?></label></th>
                <td><input type="text" id="dhl_faxNumber" name="_dhl_faxNumber" value="<?php print $meta->faxNumber  ?>" class="regular-text" /></span></td>
            </tr>  
            <?php endif; ?>
            <?php if($this->settings->enable_email): ?>                    
            <tr valign="top">
                <th scope="row"><label for="_dhl_email"><?php _e('Email', 'dh_localseo');?></label></th>
                <td><input type="text" id="dhl_email" name="_dhl_email" value="<?php print $meta->email  ?>" class="regular-text" /></td>
            </tr> 
            <?php endif; ?>
            <?php if($this->settings->enable_url): ?>                   
            <tr valign="top">
                <th scope="row"><label for="_dhl_url"><?php _e('Website', 'dh_localseo');?></label></th>
                <td><input type="text" id="dhl_url" name="_dhl_url" value="<?php print $meta->url  ?>" class="regular-text" /></td>
            </tr> 
            <?php endif; ?>
            <?php if($this->settings->enable_description): ?>       
            <tr valign="top">
                <th scope="row"><label for="_dhl_description"><?php _e('Description', 'dh_localseo');?></label></th>
                <td><textarea id="dhl_description" style="width: 25em" rows="7" name="_dhl_description" ><?php print $meta->description ?></textarea><br/>
                    <span class="howto"><?php _e('HTML is allowed, linebreaks are set automatically.', 'dh_localseo');?></span></td>
            </tr>  
            <?php endif; ?>
            <?php if($this->settings->enable_openingHours): ?>       
            <tr valign="top">
                <th scope="row"><label for="_dhl_openingHours"><?php _e('Opening hours', 'dh_localseo');?></label></th>  
                <td>
            <?php
            $openingHours = $meta->openingHours;

      ?> 
                    <div id="openingHours">
                    <div id="openingHours_template"  class="openingHours_entity" style="display:none">
                      <select class="day">
                        <option value="1">Montag</option>
                        <option value="2">Dienstag</option>
                        <option value="3">Mittwoch</option>
                        <option value="4">Donnerstag</option>
                        <option value="5">Freitag</option>
                        <option value="6">Samstag</option>
                        <option value="7">Sonntag</option>
                      </select>
                      <input type="text" class="textfield from_hr mini" maxlength="2">:<input type="text" class="textfield from_min mini" maxlength="2"> bis <input type="text" class="textfield to_hr mini" maxlength="2">:<input type="text" class="textfield to_min mini" maxlength="2">
                      <a class="del" href="#">[x]</a>
                    </div>
                    <?php      
                    $counter = 0; if(is_array($openingHours) && !empty($openingHours)): foreach ($openingHours as $k => $t_inner): if(is_array($t_inner) && !empty($t_inner)): foreach ($t_inner as  $kk => $v):  ?>
                    <div class="openingHours_entity">
                      <select class="day" name="_dhl_openingHours[<?php print $k?>][<?php print $kk?>][day]">
                        <option value="1"<?php if ($v['day'] == 1) print ' selected="selected"'; ?>>Montag</option>
                        <option value="2"<?php if ($v['day'] == 2) print ' selected="selected"'; ?>>Dienstag</option>
                        <option value="3"<?php if ($v['day'] == 3) print ' selected="selected"'; ?>>Mittwoch</option>
                        <option value="4"<?php if ($v['day'] == 4) print ' selected="selected"'; ?>>Donnerstag</option>
                        <option value="5"<?php if ($v['day'] == 5) print ' selected="selected"'; ?>>Freitag</option>
                        <option value="6"<?php if ($v['day'] == 6) print ' selected="selected"'; ?>>Samstag</option>
                        <option value="7"<?php if ($v['day'] == 7) print ' selected="selected"'; ?>>Sonntag</option>
                      </select>
                      <input name="_dhl_openingHours[<?php print $k?>][<?php print $kk?>][from_hr]" type="text" class="textfield from_hr mini" maxlength="2" value="<?php print $v['from_hr']?>">:<input name="_dhl_openingHours[<?php print $k?>][<?php print $kk?>][from_min]" type="text" class="textfield from_min mini" maxlength="2" value="<?php print $v['from_min']?>"> bis <input name="_dhl_openingHours[<?php print $k?>][<?php print $kk?>][to_hr]" type="text" class="textfield to_hr mini" maxlength="2" value="<?php print $v['to_hr']?>">:<input name="_dhl_openingHours[<?php print $k?>][<?php print $kk?>][to_min]" type="text" class="textfield to_min mini" maxlength="2" value="<?php print $v['to_min']?>">
                      <a class="del" href="#">[x]</a>
                    </div>             
                    <?php $counter++; endforeach;endif;endforeach;endif; ?>
                    <a id="add_openingHours" href="#">Öffnungszeit hinzufügen</a><br/>
                    <style type="text/css">.mini { width: 30px} .error {border-color: red!important} .message_error2 {color: red}</style>
                    <script type="text/javascript">
                      jQuery(document).ready(function() {
                      
                        jQuery('.openingHours_entity .day').change(function() {
                            var day = jQuery(this);
                            day.attr('name',  jQuery(this).attr('name').replace(/openingHours\[[0-9]{1}\]/, 'openingHours['+day.val()+']'));
                            day.parent().find('input').each(function(){jQuery(this).attr('name',  jQuery(this).attr('name').replace(/openingHours\[[0-9]{1}\]/, 'openingHours['+day.val()+']'))});
                        });
                          
                        jQuery('.openingHours_entity .del').click(function() {
                          jQuery(this).parent().remove();
                          return false;
                        });
                     
                      
                        var counter = <?php print $counter ?>;
                        jQuery('#add_openingHours').click(function() {
                          openingHours_entity = jQuery('#openingHours_template').clone();
                          //openingHours_entity.Attr('id', 'entity-'+$counter);
                          openingHours_entity.find('.day').attr('name', '_dhl_openingHours[1]['+counter+'][day]');
                          openingHours_entity.find('.day').change(function() {
                            var day = jQuery(this);
                            day.attr('name',  jQuery(this).attr('name').replace(/_dhl_openingHours\[[0-9]{1}\]/, '_dhl_openingHours['+day.val()+']'));
                            day.parent().find('input').each(function(){jQuery(this).attr('name',  jQuery(this).attr('name').replace(/_dhl_openingHours\[[0-9]{1}\]/, '_dhl_openingHours['+day.val()+']'))});
                          });
                          openingHours_entity.find('.from_hr').attr('name', '_dhl_openingHours[1]['+counter+'][from_hr]');
                          openingHours_entity.find('.from_min').attr('name', '_dhl_openingHours[1]['+counter+'][from_min]');
                          openingHours_entity.find('.to_hr').attr('name', '_dhl_openingHours[1]['+counter+'][to_hr]');
                          openingHours_entity.find('.to_min').attr('name', '_dhl_openingHours[1]['+counter+'][to_min]');
     
                          openingHours_entity.find('.del').click(function() {
                            jQuery(this).parent().remove();
                            return false;
                          });                      
     
                          
                          jQuery('#add_openingHours').before(openingHours_entity.show()); 
                                              
                          //On Keypress
                          jQuery(this).find('.from_min, .to_min, .from_hr, .to_hr').keyup(function(){validate_openingHours()});

                          //On Blur
                           jQuery(this).find('.from_min, .to_min, .from_hr, .to_hr').blur(function(){validate_openingHours()});
                          
                          counter++;
                          return false;                     
                        });
                        
                        
                        //On Keypress
                        jQuery('.from_min, .to_min, .from_hr, .to_hr').keyup(function(){validate_openingHours()});

                        //On Blur
                        jQuery('.from_min, .to_min, .from_hr, .to_hr').blur(function(){validate_openingHours()});
                                        
                        
                        function validate_openingHours()
                        {
                          var myerr = 0;
                          
                         
                          jQuery('.openingHours_entity:not(#openingHours_template)').each(function() {
                            var entity = jQuery(this);
                            entity.find('.from_hr, .to_hr').each(function() {
                            
                              if (jQuery(this).val() == '' || isNaN(jQuery(this).val()) || jQuery(this).val() < 0 || jQuery(this).val() > 24) {
                                jQuery(this).addClass('error');
                                myerr++;
                              }else
                                jQuery(this).removeClass('error');
                                
                            });
                            entity.find('.from_min, .to_min').each(function() {
                              if (jQuery(this).val() == '' || isNaN(jQuery(this).val()) || jQuery(this).val() < 0 || jQuery(this).val() > 59) {
                                jQuery(this).addClass('error');
                                myerr++;
                              } else
                                 jQuery(this).removeClass('error');
                            });     
                          });

                          if (myerr > 0) {
                            jQuery('#property_openingHours_span').text("Bitte überprüfen Sie die Zeitangaben.");
                            jQuery('#property_openingHours_span').addClass("message_error2");  
                            return false;
                                
                          }
                          else {
                            jQuery('#property_openingHours_span').text("");
                            jQuery('#property_openingHours_span').removeClass("message_error2");  
                            return true;
                          }
                        }
                        
                        
                      });
                      function setNameperDay(entity, count) {
                        jQuery(entity).attr('name', '_dhl_openingHours['+jQuery(this).val()+']['+count+'][day]');
                      }
                    </script>
                  </div>    
                  <span class="message_error2" id="property_openingHours_span"></span>
              </td>
            </tr>
            <?php endif; ?>
        </table>
    <?php

    }

    /**
     * save_meta_boxes
     * saves meta box inputs
     *
     * @since 1.0.
     *
     */
     
    function save_meta_boxes( $post_id ) {
        
        // First we need to check if the current user is authorised to do this action. 
        if ( 'page' == $_POST['post_type'] ) {
            if ( ! current_user_can( 'edit_page', $post_id ) )
                return;
        } else {
            if ( ! current_user_can( 'edit_post', $post_id ) )
                return;
        }

        // Secondly we need to check if the user intended to change this value.
        if ( ! isset( $_POST['dh_localseo_nonce'] ) || ! wp_verify_nonce( $_POST['dh_localseo_nonce'], plugin_basename( __FILE__ ) ) )
            return;

        // Thirdly we can save the value to the database

        //if saving in a custom table, get post_ID
        $post_ID = $_POST['post_ID'];

        foreach ($this->fields as $field => $name) {
           
            if ($field == "openingHours") {
                if (!empty($_POST[$name])) {
                    ksort($_POST[$name]);
                    foreach ($_POST[$name] as $k => $v) {
                        usort($v, array('DHLocalSeo', 'cmp_time'));
                        $_POST[$name][$k] = $v;
                    }
                }
            }
           
            update_post_meta( $post_ID, $name, $_POST[$name]);
        }
    }
    
    
    /**
     * show_location_template
     * Shows the location data of a post
     *
     * @since 1.0.
     *
     * @include includes the html template either from plugin or from theme root
     *
     * @return formated html of location data
     *
     */

    function show_location_template() {
        global $post;
       
       
        ob_start();
        if (file_exists(get_stylesheet_directory().'/location-template.php'))
            include (get_stylesheet_directory().'/location-template.php');
        else
            include(plugin_dir_path(__FILE__).'/location-template.php');
        
        $output = ob_get_contents();
        ob_clean();
        return $output;
    }
    
    /**
     * the_content
     * Adds the location data to "the_content" filter
     *
     * @since 1.0.     
     *
     * @return content (html) for the_content filter    
     *
     */    
    
    function the_content($content) {
        $content .= $this->show_location_template();
        return $content;
    }
    
    // [bartag foo="foo-value"]
    function shortcode( $atts ) {
        return $this->show_location_template();
    }
   
    
    /**
     * openingHours
     * creates opening hours table
     *
     * @since 1.0.
     *
     * @return formated html-table with all opening hours
     */    
    
    function openingHours ($timing) {
        if (is_array($timing)) { 
            $output = '<table class="dhl_openingHours">';
            foreach ($timing as $k => $t_inner) {
                $output .= '<tr><td><strong>';
                    switch ($k) {
                      case 1: $dtday = "Mo"; $day = __("Monday", 'dhl_localseo');break;
                      case 2: $dtday = "Tu"; $day = __("Tuesday", 'dhl_localseo');break;
                      case 3: $dtday = "We"; $day = __("Wednesday", 'dhl_localseo');break;
                      case 4: $dtday = "Th"; $day = __("Thursday", 'dhl_localseo');break;
                      case 5: $dtday = "Fr"; $day = __("Friday", 'dhl_localseo');break;
                      case 6: $dtday = "Sa"; $day = __("Saturday", 'dhl_localseo');break;
                      case 7: $dtday = "So"; $day = __("Sunday", 'dhl_localseo');break;
                    } 
                $output .= $day.'</strong></td><td>';
                foreach ($t_inner as $v) {
                  $output .= '<meta  itemprop="openingHours" content="'.$dtday.' '.$v['from_hr'].':'.$v['from_min'].'-'.$v['to_hr'].':'.$v['to_min'].'"/>'.$v['from_hr'].':'.$v['from_min'].' - '.$v['to_hr'].':'.$v['to_min'].'<br/>';
                }
                $output .= '</td></tr>';
                
            } 
            $output .= '</table>';
        }
        
        return $output;
    }
    
    /**
     * cmp_time
     * helper function for ksort to compare opening hours
     *
     * @since 1.0.
     *
     * @return 0,1 or -1
     */     
    function cmp_time($a, $b)
    {
        if ($a['from_min'].$a['from_min'] == $b['from_min'].$b['from_min']) {
            return 0;
        }
        return ($a['from_hr'].$a['from_min'] < $b['from_min'].$b['from_min']) ? -1 : 1;
    }
    

    /**
     * create_sitemap
     * Creates XML-Sitemap with included kml file
     *
     * @since 1.0.
     *
     *
     */      

    function create_sitemap () {
        global $wpdb;
        
        //get which post types are to be considered
        
        $count = $wpdb->get_var( $wpdb->prepare( "SELECT COUNT(ID) FROM $wpdb->posts WHERE post_type = %s AND post_status = 'publish'", 'location' ) );
        

        echo '<?xml version="1.0" encoding="UTF-8"?>
        <urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:geo="http://www.google.com/geo/schemas/sitemap/1.0">';
        
        for ($i = 0; $i <= ($count/1000);$i++) {
            
            echo ' 
            <url>
                <loc>'.home_url( 'locations'.($i+1).'.kml' ).'</loc>
            </url>';
        }
        echo '</urlset>
        ';
        die();
    }
    
    
   /**
     * create_kml
     * Creates KML-Sitemap with geodata
     *
     * @since 1.0.
     *
     *
     */      

    function create_kml () {
        global $wpdb,$wp_query;
        
        
        $n = (int) $wp_query->query_vars['kml_n'];
        $offset = ($n > 1) ? ($n - 1) * 1000 : 0;
        $total = $offset + 1000;

        
        echo '<?xml version="1.0" encoding="UTF-8"?>
        <kml xmlns="http://www.opengis.net/kml/2.2" xmlns:atom="http://www.w3.org/2005/Atom">
            <Document>
                <name>'.get_bloginfo('name').'</name>
                <atom:link rel="related" href="'.home_url().'" />
                <Folder>';
        
        $query = "SELECT ID, post_title, post_content, post_name
            FROM $wpdb->posts p
            JOIN $wpdb->postmeta pm
            ON pm.post_id = p.ID
            WHERE p.post_status = 'publish'
            AND	p.post_password = ''
            AND p.post_type in ('".implode("','", $this->get_enabled_post_types())."')
            AND (pm.meta_key = '_dhl_latitude' AND pm.meta_value != 0 AND pm.meta_value != '')
            LIMIT 1000 OFFSET $offset";        

        $posts = $wpdb->get_results($query);
        
        $srcharr = array("&#038;"); 
        $replarr = array("&");

        foreach ( $posts as $p ) {
               
               
            echo '      <Placemark id="'.$p->post_name.'">
                            <name><![CDATA['.get_post_meta($p->ID, '_dhl_name', true).']]></name>
                            <address><![CDATA['.get_post_meta($p->ID, '_dhl_streetAddress', true).', '.get_post_meta($p->ID, '_dhl_postalCode', true).' '.get_post_meta($p->ID, '_dhl_addressLocality', true).']]></address>
                            <description><![CDATA[';
                                ?><?php if (get_post_meta($p->ID,'_dhl_name',true)): ?>
                                    <h2 class="dhl_name" itemprop="name" ><?php print get_post_meta($p->ID, '_dhl_name', true); ?></h2>
                                <?php endif; ?>
                                 <?php if (get_post_meta($p->ID,'_dhl_streetAddress',true) || get_post_meta($p->ID,'_dhl_postalCode',true) || get_post_meta($p->ID,'_dhl_addressLocality',true) ): ?>
                                <p class="dhl_address" itemprop="address" itemscope itemtype="http://schema.org/PostalAddress">
                                    <?php if (get_post_meta($p->ID,'_dhl_streetAddress',true)): ?><span class="dhl_streetAddress" itemprop="streetAddress"><?php print get_post_meta($p->ID,'_dhl_streetAddress',true) ?></span><br/><?php endif; ?>
                                    <?php if (get_post_meta($p->ID,'_dhl_postalCode',true)): ?><span class="dhl_postalCode" itemprop="postalCode"><?php print get_post_meta($p->ID,'_dhl_postalCode',true) ?></span><?php endif; if (get_post_meta($p->ID,'_dhl_addressLocality',true)): ?> <span  class="dhl_addressLocality" itemprop="addressLocality"><?php print get_post_meta($p->ID,'_dhl_addressLocality',true) ?></span><?php endif; ?>
                                </p>  
                                <?php endif; ?>    
                                
                                <?php if (get_post_meta($p->ID,'_dhl_telephone',true) && $this->settings->enable_telephone):?>
                                    <h2><?php _e('Phone', 'dh_localseo');?></h2>
                                    <p class="dhl_telephone" itemprop="telephone"><?php echo get_post_meta($p->ID,'_dhl_telephone',true);?></p>
                                <?php endif; ?>
                                
                                <?php if (get_post_meta($p->ID,'_dhl_faxNumber',true) && $this->settings->enable_faxNumber):?>
                                    <h2><?php _e('Fax', 'dh_localseo');?></h2>
                                    <p class="dhl_faxNumber" itemprop="faxNumber"><?php echo get_post_meta($p->ID,'_dhl_faxNumber',true);?></p>        
                                <?php endif; ?>
                                
                                <?php if (get_post_meta($p->ID,'_dhl_email',true) && $this->settings->enable_email):?>
                                    <h2><?php _e('Email address', 'dh_localseo');?></h2>
                                    <p class="dhl_email" itemprop="email"><a href="mailto:<?php echo get_post_meta($p->ID,'_dhl_email',true);?>"><?php echo get_post_meta($p->ID,'_dhl_email',true);?></a></p>        
                                <?php endif; ?>  
                                
                                <?php if (get_post_meta($p->ID,'_dhl_url',true) && $this->settings->enable_url):?>
                                    <h2><?php _e('Website', 'dh_localseo');?></h2>
                                    <p class="dhl_url" itemprop="url"><a href="<?php echo get_post_meta($p->ID,'_dhl_url',true);?>" target="_blank"><?php echo get_post_meta($p->ID,'_dhl_url',true);?></a></p>        
                                <?php endif; ?>                                
                                
                                <?php if (get_post_meta($p->ID,'_dhl_description',true) && $this->settings->enable_description):?>
                                    <h2><?php _e('Description', 'dh_localseo');?></h2>
                                    <p class="dhl_description" itemprop="description"><?php echo nl2br(get_post_meta($p->ID,'_dhl_description',true));?></p>        
                                <?php endif; ?>
                                
                                <?php if (get_post_meta($p->ID,'_dhl_openingHours',true)):?>
                                    <h2><?php _e('Opening hours', 'dh_localseo');?></h2>
                                    <?php print $this->openingHours(get_post_meta($p->ID,'_dhl_openingHours',true)); ?>
                                <?php endif; ?>    
            
                                <a href="<?php print get_permalink($p->ID) ?>"><?php printf(__('Detailled information on %s', 'dh_localseo'), get_bloginfo('name'))?></a></p>
<?php       echo '              ]]></description>
                            <Point>
                                <coordinates>'.get_post_meta($p->ID, '_dhl_longitude', true).','.get_post_meta($p->ID, '_dhl_latitude', true).',0</coordinates>
                            </Point>
                        </Placemark>
                        ';
        }
        echo '  </Folder>
            </Document>
        </kml>';
        die();
    }    
    
    /**
     * add_rewrite_rules
     * add rewrites to generate sitemap.xml / locations.kml
     *
     * @since 1.0.
     *
     *
     */   

    function add_rewrite_rules() {
        
        
        $GLOBALS['wp']->add_query_var( 'dhl_sitemap' );
        add_rewrite_rule( 'geo-sitemap\.xml$', 'index.php?dhl_sitemap=1', 'top' );
        
        $GLOBALS['wp']->add_query_var( 'kml' );
        $GLOBALS['wp']->add_query_var( 'kml_n' );
        add_rewrite_rule( 'locations([0-9]+)?\.kml$', 'index.php?kml=$matches[1]&kml_n=$matches[2]', 'top' );
        
        if ($_REQUEST['flush_rewrite_rules'])
            update_option('dhl_rewrite_rules', false);
        
        if (!get_option('dhl_rewrite_rules')) {
            flush_rewrite_rules();
            update_option('dhl_rewrite_rules', true);
            add_action( 'admin_notices', array($this, 'admin_notice_flush_rewrite_rules') );   
        }        
        
        
    }
    
    function admin_notice_flush_rewrite_rules() {
        ?>
        <div class="updated">
            <p><?php _e( 'The rewrite rules were flushed successfully.', 'dh_localseo' ); ?></p>
        </div>
        <?php
    }

    
    function get_enabled_post_types () {
        if ($this->settings->enable_location)
            $screens[] = 'location';
        if (!empty($this->settings->enable_post_types))foreach ($this->settings->enable_post_types as $name => $val)
            $screens[] = $name;
        return $screens;
    }
    
    
}

//create instance of class, avaiable as global variable
$dh_localseo = new DHLocalSeo();